;-------------------------------------------------------------------------------
;dos.asm
;(c) Copyright 2000-2001, Ilya Aniskovets
;-------------------------------------------------------------------------------
temp_buffer	equ 0xF800
;
%include "macros.mac"
%include "dos.h"
%include "string.asm"
;
section	.text
;
;function: _fseek
;description: set read/write position
;input:
; ix - pointer to FILEPTR structure
; hl:de - position
; a - SEEK_SET/SEEK_CUR/SEEK_END
;-------------------------------------------------------------------------------
_fseek:	exa
	ld a,h
	or l
	jz .near
	exa
	push de
	ld de,0x8000
	call .near
	call .near
	pop de
	dec hl
	jr _fseek
.near:	cp SEEK_SET
	jz .from_beg
	cp SEEK_CUR
	jz .from_cur
	cp SEEK_END
	jz .from_end
	scf
	ret
.from_beg: 	
	push hl
	ldahl (ix+FILEPTR.size)
	and a
	sbc hl,de
	stahl (ix+FILEPTR.remain)
	ldahl (ix+FILEPTR.size+2)
	jnc $+3
	dec hl
	stahl (ix+FILEPTR.remain+2)
	ld (ix+FILEPTR.curpos),e
	ld a,d
	add a,(ix+FILEPTR.sector)
	ld h,a
	and 15
	ld (ix+FILEPTR.cursector),a
        ld a,h
times 4	rra
        and #0F
        add a,(ix+FILEPTR.track)
        ld (ix+FILEPTR.curtrack),a
        pop hl
        ld a,SEEK_SET
	ret
.from_cur:
	push hl
	push bc
        ld (.stack),sp
	ld sp,ix
	pop hl
	and a
	sbc hl,de
	pop bc
	jnc $+3
	dec bc
	push bc
	push hl
	ld sp,(.stack)
	ld a,(ix+FILEPTR.curpos)
	add a,e
	ld (ix+FILEPTR.curpos),a
	ld a,d
	add a,(ix+FILEPTR.cursector)
	ld h,a
	and 15
	ld (ix+FILEPTR.cursector),a
        ld a,h
times 4	rra
        and #0F
        add a,(ix+FILEPTR.curtrack)
        ld (ix+FILEPTR.curtrack),a
        pop bc
	pop hl
	ld a,SEEK_CUR
	ret
.from_end:
	scf
	ret
;-------------------------------------------------------------------------------
.stack	dw 0
;
;function: _fread
;description: read from open file to buffer
;input:
; ix - pointer to FILEPTR structure
; bc - bytes to read
; de - pointer to buffer, sizeof(buffer)>=bytes to read
;output:
; bc - bytes, actually read from file
; if ( eof() ) zero.clear()
; else zero.set();
;-------------------------------------------------------------------------------
_fread:
	push hl,de
	ld (.toread),bc
	ld (.address),de
	ld a,(ix+FILEPTR.remain+3)
	or (ix+FILEPTR.remain+2)
	jnz .next
	ldade (ix+FILEPTR.remain)
	ld a,e
	sub c
	ld a,d
	sbc a,b
	jnc .next
	ld b,d
	ld c,e
.next	ld (.bytes),bc
.lp1:	ld a,b
	or c
	jz .finish
	ld e,(ix+FILEPTR.cursector)
	ld d,(ix+FILEPTR.curtrack)
	call _readsector
	ex de,hl
	ld e,(ix+FILEPTR.curpos)
	ld d,0
	add hl,de
	ld (.buffer),hl
	ld hl,#100
	sbc hl,de
; hl - bytes to read in this sector
	ld a,c
	sub l
	ld e,a
	ld a,b
	sbc a,h
	jc .full
	ld d,a
	ld b,h
	ld c,l
	ld hl,(#5cf4)
	ld (ix+FILEPTR.cursector),l
	ld (ix+FILEPTR.curtrack),h
.full:	exa
	push de
        ld a,(ix+FILEPTR.curpos)
	add a,c
	ld (ix+FILEPTR.curpos),a
	ld hl,(.buffer)
	ld de,(.address)
	ldir
	ld (.address),de
	pop bc
	exa
	jnc .lp1
.finish:
	ld bc,(.bytes)
	di
	ld (.stack),sp
	ld sp,ix
	pop hl
	pop de
	and a
	sbc hl,bc
	jnc $+3
	dec de
	push de
	push hl
.stack	equ $+1
	ld sp,0
        ld hl,(.toread)
	and a
	sbc hl,bc
	pop hl,de
	ret
;-------------------------------------------------------------------------------
.buffer  dw 0
.address dw 0
.toread	 dw 0
.bytes	 dw 0
;
;function: _ffind
;description: find first file by filename and type "R"
;input:
; ix - pointer to FILEPTR structure to be filled
; hl - pointer to filename, 8 chars with zero end
;output:
; carry clear and FILEPTR filled if found
; carry set if not found
;-------------------------------------------------------------------------------
_ffind:
	push hl,de,bc
	ld b,0x00
.lp1	ld a,b
	call _getfile
	jc .fail
	jz .erased
	cp "R"
	call z,_substrcompare
	jz .ok
.erased inc b
	ld a,b
	add a,a
	jnc .lp1
.fail	scf
	pop hl,de,bc
	ret
.ok	xor a
	ld l,a
	ld h,l
	ld (.allsectors),hl
	ld (ix + FILEPTR.curpos),a
	ld hl,TRDFILE.sector
	add hl,de
	ld a,(hl)
	ld (ix + FILEPTR.sector),a
	ld (ix + FILEPTR.cursector),a
	inc hl
	ld a,(hl)
	ld (ix + FILEPTR.track),a
	ld (ix + FILEPTR.curtrack),a
.loop1:
	ld hl,TRDFILE.length
	add hl,de
	ld a,(hl)
	ld (.offset),a
	inc hl
	ld a,(hl)
	ld (.length),a
	inc hl
	ld a,(hl)
	ld (.sectors),a
	inc b
	ld a,b
	call _getfile
	jc .archive
	jz .archive
	cp "!"
	jz .split
.archive:
	ld hl,(.allsectors)
.length	equ $+1
	ld de,0
	add hl,de
	ld (ix + FILEPTR.size+3),d
	ld (ix + FILEPTR.remain+3),d
	stahl (ix + FILEPTR.size+1)
	stahl (ix + FILEPTR.remain+1)
.offset equ $+1
	ld a,0
        ld (ix + FILEPTR.size+0),a
	ld (ix + FILEPTR.remain+0),a
	and a
	pop hl,de,bc
	ret
.split:	
.allsectors equ $+1
	ld hl,0
.sectors equ $+1
	ld a,0
	add a,l
	ld l,a
	ld a,h
	adc a,#00
	ld h,a
	ld (.allsectors),hl
	jr .loop1
;
;function: _getfile
;description: get description of file by index
;input:
; a - file number( catalogue index )
;output:
; if ( eof() ) carry.set();
; else {
;  carry.clear();
;  a = file type
;  de = address of file descriprion
;  if ( file.deleted() ) zero.set(); else zero.clear();
; }
;-------------------------------------------------------------------------------
_getfile:
	push hl
	ld l,a
times 4 rra
	and 0x07
	ld e,a
	ld d,0x00
	call _readsector
	ld a,l
	and 15
	ld l,a
	ld h,0x00
times 4 add hl,hl
	add hl,de
	ex de,hl
	ld hl,TRDFILE.type
	add hl,de
	ld a,(de)
	sub 0x01
	ld a,(hl)
	pop hl
	ret
;
;function: _readsector
;description: read one sector from disk
;input:
; de - track<<8 + sector
;output:
; de - pointer to buffer with sector
;-------------------------------------------------------------------------------
_readsector:
	push hl
.last:	equ $+1
	ld hl,0xFFFF
	and a
	sbc hl,de
	jr z,.read_ok
	ld (.last),de
	push ix,iy,bc
	ld iy,#5c3a
	ld hl,read_buffer
	ld bc,0x0105
	call #3d13
	pop ix,iy,bc
.read_ok:
	pop hl
	ld de,read_buffer
	ret
;
section .data
;
section .bss
;-------------------------------------------------------------------------------
read_buffer:	resb 0x100
